'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.fireOnSequence = exports.SEQ = undefined;

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };
/**
 * Created by andrew on 10/13/16.
 */
/**
 * Schedules actions with { meta: { delay: N } } to be delayed by N milliseconds.
 * Makes `dispatch` return a function to cancel the timeout in this case.
 */


var _fluxStandardAction = require('flux-standard-action');

var localNS = Symbol('SEQ');
var symNext = Symbol('next');
var symRestart = Symbol('restart');

var SEQ = exports.SEQ = {

  SINGLE: function SINGLE(token, opts) {
    var result = null;
    if (!token) {
      throw TypeError('Token expected for SEQ.SINGLE');
    }
    switch (typeof token === 'undefined' ? 'undefined' : _typeof(token)) {
      case 'string':
        result = function result(a) {
          return a !== symRestart && (a.type == token ? symNext : false);
        };
        break;
      case 'function':
        if (token.tag === localNS) {
          return token;
        }
        result = function result(a) {
          return a !== symRestart && (a.type == token.toString() ? symNext : false);
        };
        break;
      case 'object':
        // TODO: make shallow or deep comparison
        result = function result(a) {
          return a !== symRestart && (a.type == token.type ? symNext : false);
        };
        break;
    }
    if (!result) {
      throw TypeError('Token invalid type for SEQ.SINGLE');
    }

    result.toString = function () {
      return 'SINGLE:(' + token.toString() + ')';
    };
    result.tag = localNS;
    return result;
  },

  TIMES: function TIMES(token, times) {
    var _ref = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

    var strict = _ref.strict;

    token = SEQ.SINGLE(token);
    var count = 0;
    var result = function result(a) {
      if (a === symRestart) {
        count = 0;
        token(symRestart);
        return false;
      }
      var result = token(a);
      if (result === true) {
        return true;
      }

      if (result === symNext) {
        count = ++count % times;
        if (count === 0) {
          return symNext;
        } else {
          return true;
        }
      }

      if (strict) {
        count = 0;
        token(symRestart);
        return false;
      }
      return true;
    };
    result.toString = function () {
      return 'TIMES:(' + token.toString() + ' x ' + times + ')';
    };
    result.tag = localNS;
    return result;
  },

  TIMES_STRICT: function TIMES_STRICT(token, times) {
    var result = SEQ.TIMES(token, times, { strict: true });
    result.toString = function () {
      return 'TIMES_STRICT:(' + token.toString() + ' x ' + times + ')';
    };
    return result;
  },

  ALL: function ALL(tokens) {
    var _ref2 = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    var strict = _ref2.strict;

    if (!tokens || !tokens.length) {
      throw TypeError('Tokens expected for SEQ.ALL');
    }
    tokens = tokens.map(SEQ.SINGLE);
    var results = tokens.map(function (t) {
      return false;
    });
    var length = tokens.length;

    var result = function result(a) {
      if (a === symRestart) {
        results = tokens.map(function (t) {
          return false;
        });
        tokens.forEach(function (t) {
          return t(symRestart);
        });
        return false;
      }

      var nextResults = tokens.map(function (t, k) {
        var result = t(a);

        if (result === symNext) {
          return symNext;
        }
        return results[k];
      }, results);

      if (nextResults.filter(function (i) {
        return i === symNext;
      }).length === length) {
        results = tokens.map(function (t) {
          return false;
        });
        tokens.forEach(function (t) {
          return t(symRestart);
        });
        return symNext;
      }

      results = nextResults;
      return true;
    };
    result.toString = function () {
      return 'ALL:(' + tokens.map(function (t) {
        return t.toString();
      }).join() + ')';
    };
    result.tag = localNS;
    return result;
  },

  ANY: function ANY(tokens) {
    var _ref3 = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    var strict = _ref3.strict;

    if (!tokens || !tokens.length) {
      throw TypeError('Tokens expected for SEQ.ANY');
    }
    tokens = tokens.map(SEQ.SINGLE);
    var result = function result(a) {
      if (a === symRestart) {
        tokens.forEach(function (t) {
          return t(symRestart);
        });
        return false;
      }
      if (tokens.filter(function (t) {
        return t(a) === symNext;
      }).length) {
        tokens.forEach(function (t) {
          return t(symRestart);
        });
        return symNext;
      }
      if (strict) {
        tokens.forEach(function (t) {
          return t(symRestart);
        });
        return false;
      }
      return true;
    };
    result.toString = function () {
      return 'ANY:(' + tokens.map(function (t) {
        return t.toString();
      }).join() + ')';
    };
    result.tag = localNS;
    return result;
  },

  SEQUENCE: function SEQUENCE(tokens) {
    var _ref4 = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    var strict = _ref4.strict;

    if (!tokens || !tokens.length) {
      throw TypeError('Tokens expected for SEQ.SEQUENCE');
    }
    tokens = tokens.map(SEQ.SINGLE);
    var length = tokens.length;
    var count = 0;
    var result = function result(a) {
      if (a === symRestart) {
        count = 0;
        tokens.forEach(function (t) {
          return t(symRestart);
        });
        return false;
      }
      var result = tokens[count](a);
      if (result === true) {
        return true;
      }

      if (result === symNext) {
        count = ++count % length;

        if (count === 0) {
          tokens.forEach(function (t) {
            return t(symRestart);
          });
          return symNext;
        } else {
          return true;
        }
      }

      if (strict) {
        count = 0;
        tokens.forEach(function (t) {
          return t(symRestart);
        });
        return false;
      }
      return true;
    };
    result.toString = function () {
      return 'SEQUENCE:(' + tokens.map(function (t) {
        return t.toString();
      }).join() + ')';
    };
    result.tag = localNS;
    return result;
  },
  SEQUENCE_STRICT: function SEQUENCE_STRICT(tokens) {
    if (!tokens || !tokens.length) {
      throw TypeError('Tokens expected for SEQ.SEQUENCE_STRICT');
    }

    var result = SEQ.SEQUENCE(tokens, { strict: true });
    result.toString = function () {
      return 'SEQUENCE_STRICT:(' + tokens.map(function (t) {
        return t.toString();
      }).join() + ')';
    };
    return result;
  }
};

var sequences = [];

var registerSequence = function registerSequence(fn) {

  if (!fn || typeof fn !== 'function') {
    throw new TypeError('registerSequence expects function as a first parameter');
  }

  sequences.push(fn);

  // unregister
  return function () {
    var idx = sequences.indexOf(fn);
    if (idx >= 0) {
      return sequences.splice(idx, 1);
    }
  };
};

var fireOnSequence = exports.fireOnSequence = function fireOnSequence(action, sequence, _ref5) {
  var once = _ref5.once;

  return function (dispatch) {

    if (Array.isArray(sequence)) {
      sequence = SEQ.SEQUENCE(sequence);
    } else {
      sequence = SEQ.SINGLE(sequence);
    }

    console.log('SEQUENCE: ' + sequence + ' => ' + (action.type || action.toString()));

    var unregister = registerSequence(function (a) {

      var result = sequence(a);
      if (result === symNext) {

        if (once) {
          unregister();
          unregister = null;
        } else {
          sequence(symRestart);
        }

        if (typeof action == 'function') {
          dispatch(action());
        } else {
          dispatch(action);
        }
      }
    });
  };
};

exports.default = function (store) {
  return function (next) {
    return function (action) {
      var result = next(action);

      if (sequences.length && (0, _fluxStandardAction.isFSA)(action)) {
        sequences.forEach(function (s) {
          return s(action);
        });
      }

      return result;
    };
  };
};
